@@ -25,6 +25,7 @@
 
 #include "config.h"
 
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
 /* Define basic nodoka types */
@@ -719,7 +720,6 @@ create_notification(UrlClickedCb url_clicked)
 	GtkWidget *vbox;
 	GtkWidget *close_button;
 	GtkWidget *image;
-	GtkWidget *alignment;
 	AtkObject *atkobj;
 	WindowData *windata;
 	GdkVisual *visual;
@@ -808,7 +808,7 @@ create_notification(UrlClickedCb url_clicked)
 	gtk_label_set_line_wrap_mode (GTK_LABEL (windata->summary_label), PANGO_WRAP_WORD_CHAR);
 
 	atkobj = gtk_widget_get_accessible(windata->summary_label);
-	atk_object_set_description(atkobj, "Notification summary text.");
+	atk_object_set_description (atkobj, _("Notification summary text."));
 
 	/* Add the close button */
 	close_button = gtk_button_new();
@@ -822,9 +822,9 @@ create_notification(UrlClickedCb url_clicked)
 
 	atkobj = gtk_widget_get_accessible(close_button);
 	atk_action_set_description(ATK_ACTION(atkobj), 0,
-							   "Closes the notification.");
+							   _("Closes the notification."));
 	atk_object_set_name(atkobj, "");
-	atk_object_set_description(atkobj, "Closes the notification.");
+	atk_object_set_description (atkobj, _("Closes the notification."));
 
 	image = gtk_image_new_from_icon_name ("window-close", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
@@ -865,14 +865,12 @@ create_notification(UrlClickedCb url_clicked)
                          G_CALLBACK(activate_link), windata);
 
 	atkobj = gtk_widget_get_accessible(windata->body_label);
-	atk_object_set_description(atkobj, "Notification body text.");
-
-	alignment = gtk_alignment_new(1, 0.5, 0, 0);
-	gtk_widget_show(alignment);
-	gtk_box_pack_start(GTK_BOX(vbox), alignment, FALSE, TRUE, 0);
+	atk_object_set_description (atkobj, _("Notification body text."));
 
 	windata->actions_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
-	gtk_container_add(GTK_CONTAINER(alignment), windata->actions_box);
+	gtk_widget_set_halign (windata->actions_box, GTK_ALIGN_END);
+	gtk_widget_show(windata->actions_box);
+	gtk_box_pack_start(GTK_BOX(vbox), windata->actions_box, FALSE, TRUE, 0);
 
 	return GTK_WINDOW(win);
 }
@@ -882,16 +880,27 @@ void
 set_notification_text(GtkWindow *nw, const char *summary, const char *body)
 {
 	char *str;
+	char* quoted;
 	WindowData *windata = g_object_get_data(G_OBJECT(nw), "windata");
 	g_assert(windata != NULL);
 
+	quoted = g_markup_escape_text(summary, -1);
 	str = g_strdup_printf(
-		"<span color=\"#000000\"><b><big>%s</big></b></span>", summary);
+		"<span color=\"#000000\"><b><big>%s</big></b></span>", quoted);
+	g_free(quoted);
 	gtk_label_set_markup(GTK_LABEL(windata->summary_label), str);
 	g_free(str);
 
-	str = g_strdup_printf(
-        "<span color=\"#000000\">%s</span>", body);
+	if (strstr(body, "&amp;") || strstr(body, "&lt;") || strstr(body, "&gt;") || strstr(body, "&apos;") || strstr(body, "&quot;"))
+	{
+		str = g_strdup_printf("<span color=\"#000000\">%s</span>", body);
+	}
+	else
+	{
+		quoted = g_markup_escape_text(body, -1);
+		str = g_strdup_printf("<span color=\"#000000\">%s</span>", quoted);
+		g_free(quoted);
+	}
 	gtk_label_set_markup(GTK_LABEL(windata->body_label), str);
 	g_free(str);
 
@@ -963,21 +972,16 @@ add_notification_action(GtkWindow *nw, const char *text, const char *key,
 
 	g_assert(windata != NULL);
 
-	if (!gtk_widget_get_visible(windata->actions_box))
+	if (gtk_widget_get_visible(windata->actions_box))
 	{
-		GtkWidget *alignment;
-
 		gtk_widget_show(windata->actions_box);
 		update_content_hbox_visibility(windata);
 
-		alignment = gtk_alignment_new(1, 0.5, 0, 0);
-		gtk_widget_show(alignment);
-		gtk_box_pack_end(GTK_BOX(windata->actions_box), alignment,
-						   FALSE, TRUE, 0);
-
 		windata->pie_countdown = gtk_drawing_area_new();
+		gtk_widget_set_halign (windata->pie_countdown, GTK_ALIGN_END);
 		gtk_widget_show(windata->pie_countdown);
-		gtk_container_add(GTK_CONTAINER(alignment), windata->pie_countdown);
+
+		gtk_box_pack_end (GTK_BOX (windata->actions_box), windata->pie_countdown, FALSE, TRUE, 0);
 		gtk_widget_set_size_request(windata->pie_countdown,
 									PIE_WIDTH, PIE_HEIGHT);
 		g_signal_connect(G_OBJECT(windata->pie_countdown), "draw",
