@@ -21,6 +21,7 @@
  */
 #include "config.h"
 
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
 typedef void (*ActionInvokedCb) (GtkWindow* nw, const char* key);
@@ -116,57 +117,81 @@ void notification_tick(GtkWindow *nw, glong remaining);
 #define BACKGROUND_OPACITY    0.92
 #define BOTTOM_GRADIENT_HEIGHT 30
 
+static void
+get_background_color (GtkStyleContext *context,
+                      GtkStateFlags    state,
+                      GdkRGBA         *color)
+{
+        GdkRGBA *c;
+
+        g_return_if_fail (color != NULL);
+        g_return_if_fail (GTK_IS_STYLE_CONTEXT (context));
+
+        gtk_style_context_get (context, state,
+                               "background-color", &c,
+                               NULL);
+
+        *color = *c;
+        gdk_rgba_free (c);
+}
+
 static void fill_background(GtkWidget* widget, WindowData* windata, cairo_t* cr)
 {
-	GtkStyle* style;
-	GdkColor* background_color;
+    GtkStyleContext *context;
+    GdkRGBA bg;
 
-	GtkAllocation allocation;
+    GtkAllocation allocation;
 
-	gtk_widget_get_allocation(widget, &allocation);
+    gtk_widget_get_allocation(widget, &allocation);
 
-	#ifdef ENABLE_GRADIENT_LOOK
+    #ifdef ENABLE_GRADIENT_LOOK
 
-		cairo_pattern_t *gradient;
-		int              gradient_y;
+        cairo_pattern_t *gradient;
+        int              gradient_y;
 
-		gradient_y = allocation.height - BOTTOM_GRADIENT_HEIGHT;
+        gradient_y = allocation.height - BOTTOM_GRADIENT_HEIGHT;
 
-	#endif
+    #endif
 
-	style = gtk_widget_get_style(widget);
-	background_color = &style->base[GTK_STATE_NORMAL];
+    context = gtk_widget_get_style_context (windata->win);
 
-	if (windata->composited)
-	{
-		cairo_set_source_rgba(cr, background_color->red / 65535.0, background_color->green / 65535.0, background_color->blue / 65535.0, BACKGROUND_OPACITY);
-	}
-	else
-	{
-		gdk_cairo_set_source_color(cr, background_color);
-	}
+    gtk_style_context_save (context);
+    gtk_style_context_set_state (context, GTK_STATE_FLAG_NORMAL);
 
-		cairo_rectangle(cr, 0, 0, allocation.width, allocation.height);
+    get_background_color (context, GTK_STATE_FLAG_NORMAL, &bg);
 
-	cairo_fill(cr);
+    gtk_style_context_restore (context);
 
-	#ifdef ENABLE_GRADIENT_LOOK
-		/* Add a very subtle gradient to the bottom of the notification */
-		gradient = cairo_pattern_create_linear(0, gradient_y, 0, allocation.height);
-		cairo_pattern_add_color_stop_rgba(gradient, 0, 0, 0, 0, 0);
-		cairo_pattern_add_color_stop_rgba(gradient, 1, 0, 0, 0, 0.15);
-		cairo_rectangle(cr, 0, gradient_y, allocation.width, BOTTOM_GRADIENT_HEIGHT);
+    if (windata->composited)
+    {
+        cairo_set_source_rgba(cr, bg.red, bg.green, bg.blue, BACKGROUND_OPACITY);
+    }
+    else
+    {
+        gdk_cairo_set_source_rgba (cr, &bg);
+    }
 
-		cairo_set_source(cr, gradient);
-		cairo_fill(cr);
-		cairo_pattern_destroy(gradient);
-	#endif
+    cairo_rectangle(cr, 0, 0, allocation.width, allocation.height);
+
+    cairo_fill(cr);
+
+    #ifdef ENABLE_GRADIENT_LOOK
+        /* Add a very subtle gradient to the bottom of the notification */
+        gradient = cairo_pattern_create_linear(0, gradient_y, 0, allocation.height);
+        cairo_pattern_add_color_stop_rgba(gradient, 0, 0, 0, 0, 0);
+        cairo_pattern_add_color_stop_rgba(gradient, 1, 0, 0, 0, 0.15);
+        cairo_rectangle(cr, 0, gradient_y, allocation.width, BOTTOM_GRADIENT_HEIGHT);
+
+        cairo_set_source(cr, gradient);
+        cairo_fill(cr);
+        cairo_pattern_destroy(gradient);
+    #endif
 }
 
 static void draw_stripe(GtkWidget* widget, WindowData* windata, cairo_t* cr)
 {
-	GtkStyle*        style;
-	GdkColor         color;
+	GtkStyleContext* context;
+	GdkRGBA bg;
 	int              stripe_x;
 	int              stripe_y;
 	int              stripe_height;
@@ -175,7 +200,9 @@ static void draw_stripe(GtkWidget* widget, WindowData* windata, cairo_t* cr)
 		double           r, g, b;
 	#endif
 
-	style = gtk_widget_get_style(widget);
+	context = gtk_widget_get_style_context (widget);
+
+	gtk_style_context_save (context);
 
 	GtkAllocation alloc;
 	gtk_widget_get_allocation(windata->main_hbox, &alloc);
@@ -193,19 +220,27 @@ static void draw_stripe(GtkWidget* widget, WindowData* windata, cairo_t* cr)
 	switch (windata->urgency)
 	{
 		case URGENCY_LOW: // LOW
-			color = style->bg[GTK_STATE_NORMAL];
+			gtk_style_context_set_state (context, GTK_STATE_FLAG_NORMAL);
+			gtk_style_context_add_class (context, GTK_STYLE_CLASS_VIEW);
+			get_background_color (context, GTK_STATE_FLAG_NORMAL, &bg);
+			gdk_cairo_set_source_rgba (cr, &bg);
 			break;
 
 		case URGENCY_CRITICAL: // CRITICAL
-			gdk_color_parse("#CC0000", &color);
+			gdk_rgba_parse (&bg, "#CC0000");
 			break;
 
 		case URGENCY_NORMAL: // NORMAL
 		default:
-			color = style->bg[GTK_STATE_SELECTED];
+			gtk_style_context_set_state (context, GTK_STATE_FLAG_SELECTED);
+			gtk_style_context_add_class (context, GTK_STYLE_CLASS_VIEW);
+			get_background_color (context, GTK_STATE_FLAG_SELECTED, &bg);
+			gdk_cairo_set_source_rgba (cr, &bg);
 			break;
 	}
 
+	gtk_style_context_restore (context);
+
 	cairo_rectangle(cr, stripe_x, stripe_y, STRIPE_WIDTH, stripe_height);
 
 	#ifdef ENABLE_GRADIENT_LOOK
@@ -220,7 +255,7 @@ static void draw_stripe(GtkWidget* widget, WindowData* windata, cairo_t* cr)
 		cairo_fill(cr);
 		cairo_pattern_destroy(gradient);
 	#else
-		gdk_cairo_set_source_color(cr, &color);
+		gdk_cairo_set_source_rgba (cr, &bg);
 		cairo_fill(cr);
 	#endif
 }
@@ -607,7 +642,6 @@ GtkWindow* create_notification(UrlClickedCb url_clicked)
 	GtkWidget* vbox;
 	GtkWidget* close_button;
 	GtkWidget* image;
-	GtkWidget* alignment;
 	AtkObject* atkobj;
 	WindowData* windata;
 
@@ -696,26 +730,25 @@ GtkWindow* create_notification(UrlClickedCb url_clicked)
 	gtk_label_set_line_wrap_mode (GTK_LABEL (windata->summary_label), PANGO_WRAP_WORD_CHAR);
 
 	atkobj = gtk_widget_get_accessible(windata->summary_label);
-	atk_object_set_description(atkobj, "Notification summary text.");
+	atk_object_set_description (atkobj, _("Notification summary text."));
 
 	/* Add the close button */
-	alignment = gtk_alignment_new(1, 0, 0, 0);
-	gtk_widget_show(alignment);
-	gtk_box_pack_start(GTK_BOX(hbox), alignment, FALSE, FALSE, 0);
-
 	close_button = gtk_button_new();
 	windata->close_button = close_button;
+	gtk_widget_set_halign (close_button, GTK_ALIGN_END);
+	gtk_widget_set_valign (close_button, GTK_ALIGN_START);
 	gtk_widget_show(close_button);
-	gtk_container_add(GTK_CONTAINER(alignment), close_button);
+	gtk_box_pack_start(GTK_BOX(hbox), close_button, FALSE, FALSE, 0);
 	gtk_button_set_relief(GTK_BUTTON(close_button), GTK_RELIEF_NONE);
 	gtk_container_set_border_width(GTK_CONTAINER(close_button), 0);
 	//gtk_widget_set_size_request(close_button, 20, 20);
 	g_signal_connect_swapped(G_OBJECT(close_button), "clicked", G_CALLBACK(gtk_widget_destroy), win);
 
 	atkobj = gtk_widget_get_accessible(close_button);
-	atk_action_set_description(ATK_ACTION(atkobj), 0, "Closes the notification.");
+	atk_action_set_description(ATK_ACTION(atkobj), 0,
+                               _("Closes the notification."));
 	atk_object_set_name(atkobj, "");
-	atk_object_set_description(atkobj, "Closes the notification.");
+	atk_object_set_description (atkobj, _("Closes the notification."));
 
 	image = gtk_image_new_from_icon_name ("window-close", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
@@ -739,6 +772,7 @@ GtkWindow* create_notification(UrlClickedCb url_clicked)
 	gtk_box_pack_start(GTK_BOX(windata->content_hbox), vbox, TRUE, TRUE, 0);
 
 	windata->body_label = gtk_label_new(NULL);
+	gtk_widget_show(windata->body_label);
 	gtk_box_pack_start(GTK_BOX(vbox), windata->body_label, TRUE, TRUE, 0);
 #if GTK_CHECK_VERSION (3, 16, 0)
 	gtk_label_set_xalign (GTK_LABEL (windata->body_label), 0.0);
@@ -752,14 +786,12 @@ GtkWindow* create_notification(UrlClickedCb url_clicked)
 	g_signal_connect(G_OBJECT(windata->body_label), "activate-link", G_CALLBACK(activate_link), windata);
 
 	atkobj = gtk_widget_get_accessible(windata->body_label);
-	atk_object_set_description(atkobj, "Notification body text.");
-
-	alignment = gtk_alignment_new(1, 0.5, 0, 0);
-	gtk_widget_show(alignment);
-	gtk_box_pack_start(GTK_BOX(vbox), alignment, FALSE, TRUE, 0);
+	atk_object_set_description (atkobj, _("Notification body text."));
 
 	windata->actions_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
-	gtk_container_add(GTK_CONTAINER(alignment), windata->actions_box);
+	gtk_widget_set_halign (windata->actions_box, GTK_ALIGN_END);
+	gtk_widget_show(windata->actions_box);
+	gtk_box_pack_start(GTK_BOX(vbox), windata->actions_box, FALSE, TRUE, 0);
 
 	return GTK_WINDOW(win);
 }
@@ -825,7 +857,16 @@ void set_notification_text(GtkWindow* nw, const char* summary, const char* body)
 	gtk_label_set_markup(GTK_LABEL(windata->summary_label), str);
 	g_free(str);
 
-	gtk_label_set_markup(GTK_LABEL(windata->body_label), body);
+	if (strstr(body, "&amp;") || strstr(body, "&lt;") || strstr(body, "&gt;") || strstr(body, "&apos;") || strstr(body, "&quot;"))
+	{
+		gtk_label_set_markup(GTK_LABEL(windata->body_label), body);
+	}
+	else
+	{
+		quoted = g_markup_escape_text(body, -1);
+		gtk_label_set_markup(GTK_LABEL(windata->body_label), quoted);
+		g_free(quoted);
+	}
 
 	if (body == NULL || *body == '\0')
 	{
@@ -892,46 +933,55 @@ void set_notification_arrow(GtkWidget* nw, gboolean visible, int x, int y)
 
 static void
 paint_countdown (GtkWidget  *pie,
-		 cairo_t    *cr,
-		 WindowData *windata)
+                 cairo_t    *cr,
+                 WindowData *windata)
 {
-	GtkStyle* style;
-	cairo_t* cr2;
-	cairo_surface_t* surface;
+    GtkStyleContext *context;
+    GdkRGBA bg;
+    GtkAllocation alloc;
+    cairo_t* cr2;
+    cairo_surface_t* surface;
 
-	style = gtk_widget_get_style(windata->win);
-	GtkAllocation alloc;
-	gtk_widget_get_allocation(pie, &alloc);
-	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
-	surface = cairo_surface_create_similar (cairo_get_target(cr),
-						CAIRO_CONTENT_COLOR_ALPHA,
-						alloc.width,
-						alloc.height);
+    context = gtk_widget_get_style_context (windata->win);
 
-	cr2 = cairo_create (surface);
+    gtk_style_context_save (context);
+    gtk_style_context_set_state (context, GTK_STATE_FLAG_SELECTED);
 
-	fill_background (pie, windata, cr2);
+    get_background_color (context, GTK_STATE_FLAG_SELECTED, &bg);
 
-	if (windata->timeout > 0)
-	{
-		gdouble pct = (gdouble) windata->remaining / (gdouble) windata->timeout;
+    gtk_style_context_restore (context);
 
-		gdk_cairo_set_source_color (cr2, &style->bg[GTK_STATE_ACTIVE]);
+    gtk_widget_get_allocation(pie, &alloc);
+    cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+    surface = cairo_surface_create_similar (cairo_get_target(cr),
+                                            CAIRO_CONTENT_COLOR_ALPHA,
+                                            alloc.width,
+                                            alloc.height);
 
-		cairo_move_to (cr2, PIE_RADIUS, PIE_RADIUS);
-		cairo_arc_negative (cr2, PIE_RADIUS, PIE_RADIUS, PIE_RADIUS, -G_PI_2, -(pct * G_PI * 2) - G_PI_2);
-		cairo_line_to (cr2, PIE_RADIUS, PIE_RADIUS);
-		cairo_fill (cr2);
-	}
+    cr2 = cairo_create (surface);
 
-	cairo_destroy(cr2);
+    fill_background (pie, windata, cr2);
 
-	cairo_save (cr);
-	cairo_set_source_surface (cr, surface, 0, 0);
-	cairo_paint (cr);
-	cairo_restore (cr);
+    if (windata->timeout > 0)
+    {
+        gdouble pct = (gdouble) windata->remaining / (gdouble) windata->timeout;
 
-	cairo_surface_destroy(surface);
+        gdk_cairo_set_source_rgba (cr2, &bg);
+
+        cairo_move_to (cr2, PIE_RADIUS, PIE_RADIUS);
+        cairo_arc_negative (cr2, PIE_RADIUS, PIE_RADIUS, PIE_RADIUS, -G_PI_2, -(pct * G_PI * 2) - G_PI_2);
+        cairo_line_to (cr2, PIE_RADIUS, PIE_RADIUS);
+        cairo_fill (cr2);
+    }
+
+    cairo_destroy(cr2);
+
+    cairo_save (cr);
+    cairo_set_source_surface (cr, surface, 0, 0);
+    cairo_paint (cr);
+    cairo_restore (cr);
+
+    cairo_surface_destroy(surface);
 }
 
 static gboolean
@@ -964,20 +1014,15 @@ void add_notification_action(GtkWindow* nw, const char* text, const char* key, A
 
 	g_assert(windata != NULL);
 
-	if (!gtk_widget_get_visible(windata->actions_box))
+	if (gtk_widget_get_visible(windata->actions_box))
 	{
-		GtkWidget* alignment;
-
 		gtk_widget_show(windata->actions_box);
 		update_content_hbox_visibility(windata);
 
-		alignment = gtk_alignment_new(1, 0.5, 0, 0);
-		gtk_widget_show(alignment);
-		gtk_box_pack_end(GTK_BOX(windata->actions_box), alignment, FALSE, TRUE, 0);
-
 		windata->pie_countdown = gtk_drawing_area_new();
+		gtk_widget_set_halign (windata->pie_countdown, GTK_ALIGN_END);
 		gtk_widget_show(windata->pie_countdown);
-		gtk_container_add(GTK_CONTAINER(alignment), windata->pie_countdown);
+		gtk_box_pack_end(GTK_BOX(windata->actions_box), windata->pie_countdown, FALSE, TRUE, 0);
 		gtk_widget_set_size_request(windata->pie_countdown, PIE_WIDTH, PIE_HEIGHT);
 		g_signal_connect (G_OBJECT (windata->pie_countdown), "draw", G_CALLBACK (on_countdown_draw), windata);
 	}
